import os
import sys
import logging


log = logging.getLogger('buildout.eggtractor')


def _find_zcml_files(package_path):

    mod_names  = os.path.basename(package_path).split('.')

    source_dir = os.path.join(package_path, 'src')
    if not os.path.exists(source_dir):
        source_dir = package_path

    inner_dir = os.path.join(source_dir, *mod_names)
    zcml_types = {'meta': '-meta', 'configure': '', 'overrides': '-overrides'}

    return [v for k, v in zcml_types.items()
              if os.path.exists(os.path.join(inner_dir, '%s.zcml' % k))]


def install(buildout=None):
    if 'tractor-src-directory' in buildout['buildout']:
        src_dirs = buildout['buildout']['tractor-src-directory'].split()
    elif os.path.exists('src'):
        src_dirs = ['src']
    else:
        return

    instance_part_names = []
    for k, v in buildout.items():
        if v.get('recipe') == 'plone.recipe.zope2instance':
            log.debug('Found Zope instance part at [%s]' % k)
            instance_part_names.append(k)
            if 'zcml' not in v:
                buildout[k]['zcml'] = ''
            if 'eggs' not in v:
                buildout[k]['eggs'] = ''

    zcml_tops = []
    if buildout['buildout'].has_key('tractor-zcml-top'):
        zcml_tops = buildout['buildout']['tractor-zcml-top'].split()
    zcml_tops.reverse()

    zcml_tops_found = {}
    for src_dir in src_dirs:
        for egg_name in os.listdir(src_dir):
            egg_path = os.path.join(src_dir, egg_name)
            #is it really an egg ?
            if os.path.exists(os.path.join(egg_path, 'setup.py')):
                log.debug('Found an egg in %s' % egg_path)

                if egg_path not in buildout['buildout']['develop']:
                    buildout['buildout']['develop'] += ' %s' % egg_path
                    log.debug("Added %s to develop section" % egg_path)

                for name in instance_part_names:
                    if egg_name not in buildout[name]['eggs']:
                        buildout[name]['eggs'] += ' %s' % egg_name
                        log.debug("Added %s to the egg section of [%s]" % (egg_name, name))

                    if egg_name in zcml_tops:
                        log.debug("Defered a top zcml: %s" % egg_name)
                        zcml_tops_found[egg_name] = egg_path
                    else:
                        for zcml_type in _find_zcml_files(egg_path):
                            zcml_slug = '%s%s' % (egg_name, zcml_type)
                            if zcml_slug not in buildout[name]['zcml']:
                                buildout[name]['zcml'] +=  ' %s' % zcml_slug
                                log.debug("Added %s to the zcml section of [%s]" % (zcml_slug, name))
    #now, handle the top zcml slugs
    for name in instance_part_names:
        #this is just for getting the right order
        top_eggs = [e for e in zcml_tops if e in zcml_tops_found.keys()]
        for egg_name in top_eggs:
            for zcml_type in _find_zcml_files(zcml_tops_found[egg_name]):
                zcml_slug = '%s%s' % (egg_name, zcml_type)
                if zcml_slug not in buildout[name]['zcml']:
                    buildout[name]['zcml'] =  '%s %s' % (zcml_slug, buildout[name]['zcml'])
                    log.debug("Added %s%s to the top of the zcml section of [%s]" % (zcml_slug, name))



"""
Generic Test case
"""
__docformat__ = 'restructuredtext'

import unittest
import doctest
import sys
import os
import shutil
import popen2
import subprocess

current_dir = os.path.abspath(os.path.dirname(__file__))

def rmdir(*args):
    dirname = os.path.join(*args)
    if os.path.isdir(dirname):
        shutil.rmtree(dirname)

def sh(cmd, in_data=None):
    _cmd = cmd
    print cmd
    p = subprocess.Popen([_cmd], shell=True, 
                         stdin=subprocess.PIPE, 
                         stdout=subprocess.PIPE, close_fds=True)

    if in_data is not None:
        p.stdin.write(in_data)
        
    p.stdin.close()
    print p.stdout.read()

def ls(*args):
    dirname = os.path.join(*args)
    if os.path.isdir(dirname):
        filenames = os.listdir(dirname)
        for filename in sorted(filenames):
            print filename
    else:
        print 'No directory named %s' % dirname

def cd(*args):
    dirname = os.path.join(*args)
    os.chdir(dirname)

def config(filename):
    return os.path.join(current_dir, filename)

def cat(*args):
    filename = os.path.join(*args)
    if os.path.isfile(filename):
        print open(filename).read()
    else:
        print 'No file named %s' % filename

def touch(*args, **kwargs):
    filename = os.path.join(*args)
    open(filename, 'w').write(kwargs.get('data',''))

#execdir = os.path.abspath(os.path.dirname(sys.executable))
tempdir = os.getenv('TEMP','/tmp')

def doc_suite(test_dir, setUp=None, tearDown=None, globs=None):
    """Returns a test suite, based on doctests found in /doctest."""
    suite = []
    if globs is None:
        globs = globals()

    flags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
             doctest.REPORT_ONLY_FIRST_FAILURE)

    package_dir = os.path.split(test_dir)[0]
    if package_dir not in sys.path:
        sys.path.append(package_dir)

    doctest_dir = test_dir

    # filtering files on extension
    docs = [os.path.join(doctest_dir, doc) for doc in
            os.listdir(doctest_dir) if doc.endswith('.txt')]

    for test in docs:
        suite.append(doctest.DocFileSuite(test, optionflags=flags, 
                                          globs=globs, setUp=setUp, 
                                          tearDown=tearDown,
                                          module_relative=False))

    return unittest.TestSuite(suite)

def test_suite():
    """returns the test suite"""
    return doc_suite(current_dir)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = "0.2"

long_description = (
    read('README.txt')
    + '\n' + 
    read('CHANGES.txt')
    + '\n' +
    'Detailed Documentation\n'
    '======================\n'
    + '\n' +
    read('buildout', 'eggtractor', 'tractor.txt')
    + '\n' +
    read('CONTRIBUTORS.txt')
    )
entry_point = 'buildout.eggtractor.tractor:install'
entry_points = {"zc.buildout.extension": ["default = %s" % entry_point]}

tests_require=['zc.buildout']

setup(name='buildout.eggtractor',
      version=version,
      description="Auto generates zcml slugs, buildout:egg and buildout:develop entries.",
      long_description=long_description,
      classifiers=[
        'Framework :: Buildout',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
      license='GPL',
      keywords='buildout extension egg tractor',
      author='Mustapha Benali',
      author_email='mustapha@headnet.dk',
      url='http://svn.plone.org/svn/collective/buildout/buildout.eggtractor',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['buildout'],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'zc.buildout'
                        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite = 'buildout.eggtractor.tests.test_suite',
      entry_points=entry_points,
      )
